-----------------------------------------------------------------------------------------
-- Virtex II FPGA configuration from a parallel flash device (Am29LV320). Configuration
-- mode is "Master SelectMAP", 8 bit parallel, CCLK is generated by Virtex II FPGA.
-- If multiple (up to 16) configuration files are stored in flash they can be selected
-- with DIP switches. Maximum CCLK clock is 8 MHz (please set in Xilinx ISE conf. options)
--
-- last mod.: 07.04.05
--
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity top_level is
	Port(	A			: out std_logic_vector(24 downto 0);		--	Flash	address
			WE_n		: out std_logic;									-- Flash write enable
			CE_n		: out std_logic;									-- Flash	chip enable
			OE_n		: out std_logic;									-- Flash	output enable
			RST_n		: out std_logic;									-- Flash reset
			BYTE_n	: out std_logic;									-- Flash	byte mode

			CCLK		: in	std_logic;									-- FPGA configuration clock
			DONE		: in	std_logic;									-- FPGA
			INIT_n	: in	std_logic;									-- FPGA

			LED_GR	: out std_logic;									-- green LED
			LED_GE	: out std_logic;									-- yellow LED
			DIP		: in std_logic_vector(3 downto 0));			-- DIP switches
end top_level;

architecture Behavioral of top_level is
	signal flash_addr: std_logic_vector(19 downto 0);
	signal flash_rst: std_logic;

begin
	A(24) <= '0' when DONE='0' else 'Z';
	A(23 downto 20) <= DIP(3 downto 0) when DONE='0' else (others => 'Z');
	A(19 downto 1) <= flash_addr(19 downto 1) when DONE='0' else (others => 'Z');
	A(0) <= not(flash_addr(0)) when DONE='0' else 'Z';

	WE_n <= '1' when DONE='0' else 'Z';
	CE_n <= '0' when DONE='0' else 'Z';
	OE_n <= '0' when DONE='0' else 'Z';
	BYTE_n <= DONE;
	RST_n <= flash_rst when DONE='0' else 'Z';

	LED_GR <= not(DONE);
	LED_GE <= DONE;

	process(CCLK, INIT_n)
	begin
		if(INIT_n = '0') then											-- async. RESET
			flash_addr <= (others =>'0');
			flash_rst <= '0';
		elsif CCLK='1' and CCLK'event then
			if INIT_n='1' then
				if flash_rst = '1' then
					flash_addr <= flash_addr + 1;						-- increment address pointer
				else
					flash_rst <= '1';										-- stop reset, start configuraion
				end if;					
			end if;
		end if;
	end process;
end Behavioral;